C *********************************************************************
C  pdfa_evo.for   Version 1.0                           Mar. 19, 2001
C
C *********************************************************************
C  Nuclear Parton Distribution Functions.
C
C  This package contains 2 sets of parton distribution functions:
C    1) TYPE I  (Cubic type distributions),
C    2) TYPE II (Quadratic type distributions).
C
C  REFERENCE:
C    "Determination of nuclear parton distributions",
C     M. Hirai, S. Kumano, and M. Miyama,
C     hep-ph/0103208, SAGA-HE-170-01, TMU-NT-01-01,
C     Submitted for publication.
C
C  QUESTION OR COMMENT TO:
C    Masanori Hirai   [hirai@hsg.phys.saga-u.ac.jp]
C    Shunzo Kumano    [kumanos@cc.saga-u.ac.jp]
C    Masanori Miyama  [miyama@comp.metro-u.ac.jp]
C ---------------------------------------------------------------------
C INPUT PARAMETERS:
C
C    ISET 1 ... TYPE I  (Cubic type distributions)
C         2 ... TYPE II (Quadratic type distributions)
C
C    IOUT 1 ... Do not write results in the output file.
C         2 ... Write results in the output file #11.
C
C    NA, NZ ... Mass number A and atomic number Z.
C               1 (nucleon) <=A<= 208 in principle. A and Z should not
C               be far away from the analyzed nuclei in the paper.
C               However, we think that the results could be
C               extrapolated into larger nuclei or nuclear matter.
C
C        NP ... Number of points at which the distribution values
C               are calculated. NP <= 3000.
C
C    Q2X(I,J) :
C      Q^2 and x values should be given in this array. PDFA_EVO
C      returns the distribution values at these (Q^2, x) points.
C      J=1 --> Q^2, J=2 --> x, and I=1 --> NP. The Q^2 and x
C      values should be arranged in ascending order.
C      Range: 1.0 GeV^2 <= Q^2 <= 10^5 GeV^2, 10^-9 <= X <= 1.0.
C ---------------------------------------------------------------------
C OUTPUT: PDF(I,IN)
C
C   The values of distribution at Q^2=Q2X(I,1) and x=Q2X(I,2).
C    IN = 1 ... u-valence quark (x u_v^A)
C         2 ... d-valence quark (x d_v^A)
C         3 ... q-bar (x q-bar^A)
C         4 ... gluon (x g^A)
C         5 ... structure function (F_2^A)
C
C   NOTE: The returned values are the distributions multiplied by x.
C *********************************************************************
      SUBROUTINE PDFA_EVO(ISET, IOUT, NA, NZ, NP, Q2X, PDF)
C ---------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NPMAX=3001, ND=4, NXMAX=1201, NTMAX=101)
      DIMENSION Q2X(NPMAX,2), PDF(NPMAX,5),
     1          RT(NPMAX), ICOUNT(NPMAX), T(NTMAX),
     2          PDFJ1(NXMAX,ND), DINTEG(NXMAX,ND)
      COMMON /FLA/F
      COMMON /COL/CF,CG,TR
      COMMON /XXX/X(NXMAX),NX
      COMMON /PPP/P(NXMAX,4), PL(NXMAX,4)
      COMMON /PPF/PLQQ,PLGG,DELQQ,DELGG
      COMMON /QQQ/PDFJ(NXMAX,ND)

      DATA NL, IDEBUG / 6, 1 /

C EVOLUTION PARAMETERS.
      DATA EPS / 1.D-14 /

      DATA ILOG, Q02, Q2, XMIN, DLAM, NF, NX, NT /
     1     2, 1.D0, 1.D5, 1.D-9, 0.1741D0, 3, 1200, 100 /

C START PDF CALCULATION.
      if(IDEBUG.eq.1) write(NL,*) 'Start PDF Calculation '

C INPUT PARAMETERS.
      if(IDEBUG.eq.1) write(NL,*) 'Get Parameters'

      IERR=0

      IF((ISET.GT.2).OR.(ISET.LT.1)) IERR=1
      IF((IOUT.GT.2).OR.(IOUT.LT.1)) IERR=2
      IF(NA.LT.1) IERR=3
      IF(NZ.LT.0) IERR=4
      IF((NA-NZ).LT.0) IERR=5

      CALL RQ2X(NP,Q2X,RT,ICOUNT,NQ2,RXMIN,RXMAX,IERR)

      IF((Q2X(1,1).LT.Q02).OR.(Q2X(NP,1).GT.Q2)) IERR=8
      IF((RXMIN.LT.XMIN).OR.(RXMAX.GT.1.D0)) IERR=9

      IF(IERR.NE.0) THEN
        WRITE(NL,1000) IERR
 1000     FORMAT(' ','***** Input Parameter Error ',I2,' *****')
        GO TO 60
      END IF

      CALL GETCOL(NF,DLAM,F,CF,CG,TR,BETA0,DLNLAM)

C T, X, AND SPLITTING FUNCTIONS ARE CALCULATED.
      if(IDEBUG.eq.1) write(NL,*) 'Get T and X'

      CALL GETT(Q02,Q2,Q2X(NP,1),NT,T,DT)
      CALL GETX(ILOG,XMIN,1.D0,RXMIN,NX,X)

      if(IDEBUG.eq.1) write(NL,*) 'Get Splitting Functions'

      CALL GETP(P,PL,PLQQ,PLGG,DELQQ,DELGG)

C INITIAL DISTRIBUTIONS ARE STORED IN PDFJ(I,IN).
      if(IDEBUG.eq.1) write(NL,*) 'Get Initial Distributions'

      CALL GETINI(ISET,NA,NZ,PDFJ)

C START Q^2 EVOLUTION.
      if(IDEBUG.eq.1) write(NL,*) 'Start Q2 Evolution'

      L=1

      ALP2PI=2.D0/BETA0/(T(1)-DLNLAM)

      DO 40 J=2,NT+1

      if(IDEBUG.eq.1) write(NL,1010) J-1, NT-1
 1010     format(' ','  Evolution Step',I5,' of',I5)

C INTEGRALS AT T=T(J)-DT ARE CALCULATED.
        CALL GETINT(DINTEG)

C DISTRIBUTIONS AT T=RT(L) ARE CALCULATED.
   20   CONTINUE
        IHIT=0
        IF(RT(L).LT.T(J)) IHIT=1
        IF(DABS(RT(L)-T(J)).LT.EPS) IHIT=1
        IF(IHIT.EQ.1) THEN
          if(IDEBUG.eq.1) write(NL,1020) Q2X(ICOUNT(L),1)
 1020       format(' ',4x,'INSERTED STEP --> Q2 =',1PE12.4,' GeV2')

          DRT=RT(L)-T(J-1)
          CALL EVOLVE(DINTEG,ALP2PI,DRT,PDFJ,PDFJ1)
          CALL GETPDF(L,ICOUNT,Q2X,PDFJ1,PDF)

          IF(L.EQ.NQ2) THEN
            if(IDEBUG.eq.1)
     1      write(NL,*) 'All the necessary calculations are finished'
            GO TO 50
          END IF

          L=L+1
          GO TO 20
        END IF

C DISTRIBUTIONS AT T=T(J) ARE CALCULATED.
        CALL EVOLVE(DINTEG,ALP2PI,DT,PDFJ,PDFJ1)

        DO 30 IN=1,ND
          DO 30 I=1,NX
            PDFJ(I,IN)=PDFJ1(I,IN)
   30   CONTINUE

        ALP2PI=2.D0/BETA0/(T(J)-DLNLAM)
   40 CONTINUE

   50 CONTINUE
      IF(IOUT.NE.1) THEN
        if(IDEBUG.eq.1) write(NL,*) 'Output Results'
        CALL OUTPDF(11,NP,Q2X,PDF)
      END IF

   60 CONTINUE

      if(IDEBUG.eq.1) write(NL,*) 'End'
      if(IDEBUG.eq.1) write(NL,*)

      END
C --------------------------------------------------------------------
      SUBROUTINE RQ2X(NP,Q2X,RT,ICOUNT,NQ2,RXMIN,RXMAX,IERR)
C --------------------------------------------------------------------
C Q2 AND X AT WHICH THE DISTRIBUTIONS ARE CALCULATED.
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NPMAX=3001)
      DIMENSION Q2X(NPMAX,2),RT(NPMAX),ICOUNT(NPMAX)

      L=1
      ICOUNT(1)=1
      RT(1)=DLOG(Q2X(1,1))
      RXMIN=Q2X(1,2)
      RXMAX=Q2X(1,2)

      DO 10 I=2,NP
        IF(Q2X(I,1).LT.Q2X(I-1,1)) THEN
          IERR=6
        ELSE IF(Q2X(I,1).GT.Q2X(I-1,1)) THEN
          L=L+1
          ICOUNT(L)=I
          RT(L)=DLOG(Q2X(I,1))
        ELSE
          IF(Q2X(I,2).LT.Q2X(I-1,2)) IERR=7
        END IF
        IF(Q2X(I,2).LT.RXMIN) RXMIN=Q2X(I,2)
        IF(Q2X(I,2).GT.RXMAX) RXMAX=Q2X(I,2)
   10 CONTINUE

      NQ2=L
      ICOUNT(NQ2+1)=NP+1

      RETURN
      END
C ---------------------------------------------------------------------
      SUBROUTINE GETCOL(NF,DLAM,F,CF,CG,TR,BETA0,DLNLAM)
C ---------------------------------------------------------------------
C GET F, COLOR FACTORS, BETA0, AND LN(LAMBDA**2).
      IMPLICIT REAL*8(A-H,O-Z)

      F=DFLOAT(NF)
      CF=4.D0/3.D0
      CG=3.D0
      TR=1.D0/2.D0
      BETA0=11.D0/3.D0*CG-4.D0/3.D0*TR*F
      DLNLAM=DLOG(DLAM*DLAM)

      RETURN
      END
C ---------------------------------------------------------------------
      SUBROUTINE GETT(Q02,Q2,RQ2MAX,NT,T,DT)
C ---------------------------------------------------------------------
C GET T AND DT.
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NTMAX=101)
      DIMENSION T(NTMAX)

      TMIN=DLOG(Q02)
      TMAX=DLOG(Q2)

      DT=(TMAX-TMIN)/DFLOAT(NT)

      DO 10 J=1,NT+1
        T(J)=TMIN+DFLOAT(J-1)*DT
   10 CONTINUE

      RTMAX=DLOG(RQ2MAX)
      NT1=ISERCH(NT+1,T,RTMAX)
      IF(NT1.EQ.NT+1) NT1=NT1-1
      NT=NT1+1

      RETURN
      END
C ---------------------------------------------------------------------
      SUBROUTINE GETX(ILOG,XMIN,XMAX,RXMIN,NX,X)
C ---------------------------------------------------------------------
C GET X.
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NXMAX=1201)
      DIMENSION X(NXMAX)

      IF(ILOG.EQ.1) THEN
        DX=(XMAX-XMIN)/DFLOAT(NX)
        DO 10 I=1,NX
          X(I)=XMIN+DFLOAT(I-1)*DX
   10   CONTINUE
        X(NX+1)=XMAX
      ELSE
        DLMAX=DLOG10(XMAX)
        DLMIN=DLOG10(XMIN)
        DX=(DLMAX-DLMIN)/DFLOAT(NX)
        DO 20 I=1,NX
          DLOGX=DFLOAT(I-1)*DX+DLMIN
          X(I)=10.D0**(DLOGX)
   20   CONTINUE
        X(NX+1)=XMAX
      END IF

      MIN=ISERCH(NX+1,X,RXMIN)
      X(MIN)=RXMIN
      IF(MIN.EQ.1) GO TO 40

      J=0
      DO 30 I=MIN,NX+1
        J=J+1
        X(J)=X(I)
   30 CONTINUE
      NX=J-1

   40 CONTINUE
      XMIN=X(1)

      RETURN
      END
C ---------------------------------------------------------------------
      SUBROUTINE GETP(P,PL,PLQQ,PLGG,DELQQ,DELGG)
C ---------------------------------------------------------------------
C CALCULATE SPLITTING FUNCTIONS.
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NXMAX=1201)
      DIMENSION P(NXMAX,4), PL(NXMAX,4)
      COMMON /FLA/F
      COMMON /COL/CF,CG,TR
      COMMON /XXX/X(NXMAX),NX

      PLQQ=2.D0*CF
      PLGG=2.D0*CG
      DELQQ=CF*3.D0/2.D0
      DELGG=2.D0*CG*(11.D0/12.D0-F*TR/CG/3.D0)

      DO 10 K=1,NX
        OMZ=1.D0-X(K)

        P(K,1)=CF*(1.D0+X(K)*X(K))/OMZ
        P(K,2)=2.D0*TR*(X(K)*X(K)+OMZ*OMZ)
        P(K,3)=CF*(1.D0+OMZ*OMZ)/X(K)
        P(K,4)=2.D0*CG*(X(K)/OMZ+OMZ/X(K)+X(K)*OMZ)

        PL(K,1)=-PLQQ/OMZ
        PL(K,2)=0.D0
        PL(K,3)=0.D0
        PL(K,4)=-PLGG/OMZ
   10 CONTINUE

      DO 20 IP=1,4
        P(NX+1,IP)=0.D0
        PL(NX+1,IP)=0.D0
   20 CONTINUE

      RETURN
      END
C ---------------------------------------------------------------------
      SUBROUTINE GETINI(ISET,NA,NZ,PDFJ)
C ---------------------------------------------------------------------
C GET INITIAL DISTRIBUTIONS.
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NXMAX=1201, ND=4)
      DIMENSION PDFJ(NXMAX,ND), PRMA(4), PDFN(4), WF(4)
      COMMON /XXX/X(NXMAX),NX

      A=DFLOAT(NA)
      Z=DFLOAT(NZ)
      ZA=Z/A
      OA=1.D0-1.D0/A**(1.D0/3.D0)

      CALL GETAVG(ISET,ZA,PRMA(1),PRMA(2),PRMA(4))

      DO 10 I=1,NX
        CALL WFUN(ISET,OA,PRMA,X(I),WF)
        CALL FMRST(X(I),PDFN)
        UVA=WF(1)*(ZA*PDFN(1)+(1.D0-ZA)*PDFN(2))
        DVA=WF(2)*((1.D0-ZA)*PDFN(1)+ZA*PDFN(2))
        QBA=WF(3)*PDFN(3)
        GLA=WF(4)*PDFN(4)
        PDFJ(I,1)=UVA+DVA+6.D0*QBA
        PDFJ(I,2)=GLA
        PDFJ(I,3)=DVA+2.D0*QBA
        PDFJ(I,4)=2.D0*QBA
   10 CONTINUE

      DO 20 IN=1,ND
        PDFJ(NX+1,IN)=0.D0
   20 CONTINUE

      RETURN
      END
C ---------------------------------------------------------------------
      SUBROUTINE GETAVG(IS,ZA,AUV,ADV,AG)
C ---------------------------------------------------------------------
C GET PARAMETERS AUV, ADV, AND AG.
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION DI(8,2)

      DATA ((DI(I,J), I=1,8), J=1,2) /
     1     -7.990D-04,  8.540D-03,  2.406D+00,  1.148D+00,
     2      4.777D-01,  1.772D-01,  8.326D-02,  5.246D-01,
     3     -1.474D-01, -5.297D-02,  3.768D+00,  1.583D+00,
     4      1.157D+00,  3.629D-01, -7.650D-03,  5.246D-01 /

      OZA=1.D0-ZA

      AUV=-(ZA*DI(1,IS)+OZA*DI(2,IS))/(ZA*DI(3,IS)+OZA*DI(4,IS))
      ADV=-(OZA*DI(1,IS)+ZA*DI(2,IS))/(OZA*DI(3,IS)+ZA*DI(4,IS))
      AG=-(AUV*(ZA*DI(5,IS)+OZA*DI(6,IS))
     1    +ADV*(OZA*DI(5,IS)+ZA*DI(6,IS))+DI(7,IS))/DI(8,IS)

      RETURN
      END
C ------------------------------------------------------------------
      SUBROUTINE WFUN(ISET,OA,PRMA,X,WF)
C ------------------------------------------------------------------
C FUNCTIONS W(X,A,Z).
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION PRMA(4), PRMW(4,4,2), AQB(2), WF(4)

      DATA (((PRMW(I,IN,IS), I=1,4), IN=1,4), IS=1,2) /
     1      6.222D-01, -2.858D+00,  2.557D+00,  8.107D-01,
     2      6.222D-01, -2.858D+00,  2.557D+00,  8.107D-01,
     3      6.995D+00, -3.417D+01,  6.254D+01,  1.000D+00,
     4      8.008D-01, -4.004D-01,  0.000D+00,  1.000D+00,
     1     -2.593D-01,  2.586D-01,  0.000D+00,  2.108D+00,
     2     -2.593D-01,  2.586D-01,  0.000D+00,  2.108D+00,
     3      3.744D+00, -2.236D+00,  0.000D+00,  1.000D+00,
     4      4.798D-01, -2.399D-01,  0.000D+00,  1.000D+00 /

      DATA AQB(1), AQB(2) / -0.3313D0, -0.2900D0 /

      PRMA(3)=AQB(ISET)

      DO 10 IN=1,4
        WD=PRMA(IN)+PRMW(1,IN,ISET)*X+PRMW(2,IN,ISET)*X*X
     1     +PRMW(3,IN,ISET)*X*X*X
        WF(IN)=1.D0+OA*WD/((1.D0-X)**PRMW(4,IN,ISET))
   10 CONTINUE

      RETURN
      END
C ------------------------------------------------------------------
      SUBROUTINE FMRST(X,PDFN)
C ------------------------------------------------------------------
C INITIAL DISTRIBUTIONS IN THE NUCLEON.
C MRST98 LO (central gluon), Eur. Phys. J. C4, 463 (1998).
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION PRM(5,4), PDFN(4)

      DATA ((PRM(I,IN), I=1,5), IN=1,4) /
     1  4.779D-01,  3.012D-01,  3.177D+00, -4.085D-01,  1.760D+01,
     2  3.325D-02,  1.535D-01,  3.398D+00,  3.725D+01,  3.114D+01,
     3  4.915D-01, -1.899D-01,  8.653D+00, -9.052D-01,  1.153D+01,
     4  6.107D+00,  1.257D-01,  5.511D+00, -4.255D+00,  7.274D+00 /

C IN ... 1 -> UV    DISTRIBUTION
C        2 -> DV    DISTRIBUTION
C        3 -> SEA   DISTRIBUTION
C        4 -> GLUON DISTRIBUTION

      DO 10 IN=1,4
        PDFN(IN)=PRM(1,IN)*(X**PRM(2,IN))*((1.D0-X)**PRM(3,IN))
     1           *(1.D0+PRM(4,IN)*DSQRT(X)+PRM(5,IN)*X)
   10 CONTINUE

      PDFN(3)=PDFN(3)/6.D0

      RETURN
      END
C ---------------------------------------------------------------------
      SUBROUTINE GETINT(DINTEG)
C ---------------------------------------------------------------------
C INTEGRALS AT T=T(J)-DT ARE CALCULATED.
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NXMAX=1201, ND=4)
      DIMENSION DINTEG(NXMAX,ND)
      EXTERNAL FUNP
      COMMON /FLA/F
      COMMON /XXX/X(NXMAX),NX
      COMMON /PPF/PLQQ,PLGG,DELQQ,DELGG
      COMMON /QQQ/PDFJ(NXMAX,ND)
      COMMON /IPQ/B(NXMAX,ND),C(NXMAX,ND),D(NXMAX,ND)

      DO 10 IN=1,ND
        CALL SPLINE(NX+1,X,PDFJ,B,C,D,IN)
   10 CONTINUE

      DO 20 I=1,NX
        DLNO=DLOG(1.D0-X(I))
        PDQQ=PLQQ*DLNO+DELQQ
        PDGG=PLGG*DLNO+DELGG

        QGPART=BF(FUNP,I,2,2)
        DINTEG(I,1)=BF(FUNP,I,1,1)+F*QGPART+PDQQ*PDFJ(I,1)
        DINTEG(I,2)=BF(FUNP,I,3,1)+BF(FUNP,I,4,2)+PDGG*PDFJ(I,2)
        DINTEG(I,3)=BF(FUNP,I,1,3)+QGPART+PDQQ*PDFJ(I,3)
        DINTEG(I,4)=BF(FUNP,I,1,4)+QGPART+PDQQ*PDFJ(I,4)
   20 CONTINUE

      DO 30 IN=1,ND
        DINTEG(NX+1,IN)=0.D0
   30 CONTINUE

      RETURN
      END
C ---------------------------------------------------------------------
      REAL*8 FUNCTION FUNP(K,I,IP,IN)
C ---------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NXMAX=1201, ND=4)
      COMMON /XXX/X(NXMAX),NX
      COMMON /PPP/P(NXMAX,4), PL(NXMAX,4)
      COMMON /QQQ/PDFJ(NXMAX,ND)
      COMMON /IPQ/B(NXMAX,ND),C(NXMAX,ND),D(NXMAX,ND)

      Y=X(I)/X(K)

      PDFY=SEVAL(NX+1,Y,X,PDFJ,B,C,D,IN)

      FUNP=P(K,IP)*PDFY+PL(K,IP)*PDFJ(I,IN)

      RETURN
      END
C ---------------------------------------------------------------------
      REAL*8 FUNCTION BF(FUN,I,IP,IN)
C ---------------------------------------------------------------------
C CALCULATE INTEGRAL(S) BY BRUTE-FORCE METHOD.
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NXMAX=1201)
      COMMON /XXX/X(NXMAX),NX

      SUM=0.D0

      DO 10 K=I,NX
        DZ=X(K+1)-X(K)
        F1=FUN(K,I,IP,IN)
        SUM=SUM+DZ*F1
   10 CONTINUE

      BF=SUM

      RETURN
      END
C ---------------------------------------------------------------------
      SUBROUTINE EVOLVE(DINTEG,ALP2PI,DT,PDFJ,PDFJ1)
C ---------------------------------------------------------------------
C DISTRIBUTIONS AT T=T(J) OR AT T=RT(L) ARE CALCULATED.
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NXMAX=1201, ND=4)
      DIMENSION DINTEG(NXMAX,ND), PDFJ(NXMAX,ND), PDFJ1(NXMAX,ND)
      COMMON /XXX/X(NXMAX),NX

      DO 10 IN=1,ND
        DO 10 I=1,NX+1
          PDFJ1(I,IN)=PDFJ(I,IN)+DT*ALP2PI*DINTEG(I,IN)
   10 CONTINUE

      RETURN
      END
C --------------------------------------------------------------------
      SUBROUTINE GETPDF(L,ICOUNT,Q2X,PDFJ1,PDF)
C --------------------------------------------------------------------
C DISTRIBUTIONS AT GIVEN Q^2-X POINTS ARE CALCULATED.
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NPMAX=3001, NXMAX=1201, ND=4)
      DIMENSION ICOUNT(NPMAX), Q2X(NPMAX,2),
     1          PDFJ1(NXMAX,ND), RPDF(ND), PDF(NPMAX,5),
     2          BB(NXMAX,ND), CC(NXMAX,ND), DD(NXMAX,ND)
      COMMON /XXX/X(NXMAX),NX

      DO 10 IN=1,ND
        CALL SPLINE(NX+1,X,PDFJ1,BB,CC,DD,IN)
   10 CONTINUE

      DO 30 I=ICOUNT(L),ICOUNT(L+1)-1
        DO 20 IN=1,ND
          RPDF(IN)=SEVAL(NX+1,Q2X(I,2),X,PDFJ1,BB,CC,DD,IN)
   20   CONTINUE
        PDF(I,1)=RPDF(1)-RPDF(3)-2.D0*RPDF(4)
        PDF(I,2)=RPDF(3)-RPDF(4)
        PDF(I,3)=RPDF(4)/2.D0
        PDF(I,4)=RPDF(2)
        PDF(I,5)=(4.D0*RPDF(1)-3.D0*(RPDF(3)+RPDF(4)))/9.D0
   30 CONTINUE

      RETURN
      END
C --------------------------------------------------------------------
      SUBROUTINE OUTPDF(NFILE,NP,Q2X,PDF)
C --------------------------------------------------------------------
C CALCULATED RESULTS AT GIVEN Q^2-X POINTS ARE WRITTEN IN OUTPUT
C FILES.
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NPMAX=3001)
      DIMENSION Q2X(NPMAX,2), PDF(NPMAX,5)

      DO 10 I=1,NP
        WRITE(NFILE,1010) Q2X(I,1), Q2X(I,2), (PDF(I,IN), IN=1,4)
 1010     FORMAT(' ',6(1PE12.4))
        WRITE(NFILE+1,1020) Q2X(I,1), Q2X(I,2), PDF(I,5)
 1020     FORMAT(' ',3(1PE12.4))
   10 CONTINUE

      RETURN
      END
C ---------------------------------------------------------------------
      SUBROUTINE SPLINE(N,X,Y,B,C,D,IN)
C ---------------------------------------------------------------------
C CALCULATE THE COEFFICIENTS B,C,D IN A CUBIC SPLINE INTERPOLATION.
C INTERPOLATION SUBROUTINES ARE TAKEN FROM
C G.E. FORSYTHE, M.A. MALCOLM AND C.B. MOLER,
C COMPUTER METHODS FOR MATHEMATICAL COMPUTATIONS (PRENTICE-HALL, 1977).
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NXMAX=1201, ND=4)
      DIMENSION X(NXMAX),Y(NXMAX,ND),
     1          B(NXMAX,ND),C(NXMAX,ND),D(NXMAX,ND)
      NM1=N-1
      IF(N.LT.2) RETURN
      IF(N.LT.3) GO TO 250
      D(1,IN)=X(2)-X(1)
      C(2,IN)=(Y(2,IN)-Y(1,IN))/D(1,IN)
      DO 210 I=2,NM1
        D(I,IN)=X(I+1)-X(I)
        B(I,IN)=2.0D0*(D(I-1,IN)+D(I,IN))
        C(I+1,IN)=(Y(I+1,IN)-Y(I,IN))/D(I,IN)
        C(I,IN)=C(I+1,IN)-C(I,IN)
  210 CONTINUE
      B(1,IN)=-D(1,IN)
      B(N,IN)=-D(N-1,IN)
      C(1,IN)=0.0D0
      C(N,IN)=0.0D0
      IF(N.EQ.3) GO TO 215
      C(1,IN)=C(3,IN)/(X(4)-X(2))-C(2,IN)/(X(3)-X(1))
      C(N,IN)=C(N-1,IN)/(X(N)-X(N-2))-C(N-2,IN)/(X(N-1)-X(N-3))
      C(1,IN)=C(1,IN)*D(1,IN)**2.0D0/(X(4)-X(1))
      C(N,IN)=-C(N,IN)*D(N-1,IN)**2.0D0/(X(N)-X(N-3))
  215 CONTINUE
      DO 220 I=2,N
        T=D(I-1,IN)/B(I-1,IN)
        B(I,IN)=B(I,IN)-T*D(I-1,IN)
        C(I,IN)=C(I,IN)-T*C(I-1,IN)
  220 CONTINUE
      C(N,IN)=C(N,IN)/B(N,IN)
      DO 230 IB=1,NM1
        I=N-IB
        C(I,IN)=(C(I,IN)-D(I,IN)*C(I+1,IN))/B(I,IN)
  230 CONTINUE
      B(N,IN)=(Y(N,IN)-Y(NM1,IN))/D(NM1,IN)
     1        +D(NM1,IN)*(C(NM1,IN)+2.0D0*C(N,IN))
      DO 240 I=1,NM1
        B(I,IN)=(Y(I+1,IN)-Y(I,IN))/D(I,IN)
     1          -D(I,IN)*(C(I+1,IN)+2.0D0*C(I,IN))
        D(I,IN)=(C(I+1,IN)-C(I,IN))/D(I,IN)
        C(I,IN)=3.0D0*C(I,IN)
  240 CONTINUE
      C(N,IN)=3.0D0*C(N,IN)
      D(N,IN)=D(N-1,IN)
      RETURN
  250 CONTINUE
      B(1,IN)=(Y(2,IN)-Y(1,IN))/(X(2)-X(1))
      C(1,IN)=0.0D0
      D(1,IN)=0.0D0
      B(2,IN)=B(1,IN)
      C(2,IN)=0.0D0
      D(2,IN)=0.0D0
      RETURN
      END
C ---------------------------------------------------------------------
      REAL*8 FUNCTION SEVAL(N,XX,X,Y,B,C,D,IN)
C ---------------------------------------------------------------------
C CALCULATE THE DISTRIBUTION AT XX BY CUBIC SPLINE INTERPOLATION.
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NXMAX=1201, ND=4)
      DIMENSION X(NXMAX),Y(NXMAX,ND),
     1          B(NXMAX,ND),C(NXMAX,ND),D(NXMAX,ND)

      I=ISERCH(N,X,XX)

      DX=XX-X(I)
      SEVAL=Y(I,IN)+DX*(B(I,IN)+DX*(C(I,IN)+DX*D(I,IN)))

      RETURN
      END
C ---------------------------------------------------------------------
      INTEGER FUNCTION ISERCH(N,X,Y)
C ---------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NXMAX=1201)
      DIMENSION X(NXMAX)

      MIN=1
      MAX=N+1

   10 CONTINUE
      MID=(MIN+MAX)/2
      IF(Y.LT.X(MID)) THEN
        MAX=MID
      ELSE
        MIN=MID
      END IF
      IF((MAX-MIN).GT.1) GO TO 10

      ISERCH=MIN

      RETURN
      END
C *********************************************************************
C THE END OF THE PROGRAM.
C *********************************************************************
